#!/usr/bin/python3
""" Information on MPBC's MDMA Program """

from iso3166 import countries


class Session:
    """ User session information """

    def __init__(self):
        self.authorization_status = False
        self.configuration_status = False

    def is_authorized(self):
        return self.authorization_status


    def is_configured(self):
        return self.configuration_status


    def authorize(self):
        self.authorization_status = True


    def deauthorize(self):
        self.authorization_status = False



class Default:
    """ Default saved information """

    def __init__(self):
        with open('/etc/opt/adam/default.info', 'r') as d:
            study, site, participant, visit = (x.strip() for x in d.readlines()[:4])
        self.STUDY = study
        self.SITE = site
        self.PARTICIPANT = participant
        self.VISIT = visit


    def write(self, study, site, participant, visit, time, webcams, mics):
        with open('/etc/opt/adam/default.info', 'w') as d:
            d.write(study + '\n')
            d.write(site + '\n')
            d.write(participant + '\n')
            d.write(visit + '\n')
            d.write(time + '\n')
            d.write(webcams + '\n')
            d.write(mics + '\n')


class Device:

    def __init__(self):
        with open('/etc/opt/adam/device.conf', 'r') as d:
            site, ssid, pin, cstatus = (x.strip() for x in d.readlines()[:5])
        self.SITE = site
        self.SSID = ssid
        self.PIN = pin
        self.CSTATUS = cstatus
        self.NIP = None
        self.pins_match = False
        self.sidebar_data = {}
        self.show_sbar = False
        self.hide_sbar = False


    def write(self):
        """
        Opens device.conf and writes class objects to the file
        """
        with open('/etc/opt/adam/device.conf', 'w') as d:
            d.write(self.SITE + '\n')
            d.write(self.SSID + '\n')
            d.write(self.PIN + '\n')
            d.write(self.CSTATUS +'\n')


    def reset(self):
        """
        Device reset function. Sets class parameters to None type
        """
        self.SITE = None
        self.SSID = None
        self.PIN = None
        self.CSTATUS = None

        self.configure(None)


    def write_site(self, site):
        self.SITE = site
        self.configure('site')


    def write_ssid(self, ssid):
        self.SSID = ssid
        self.configure('network')


    def write_pin(self, pin):
        self.NIP = self.PIN
        self.PIN = pin
        self.configure('pin')

    
    def compare_pins(self, entered_pin):
        if entered_pin == self.PIN:
            self.pins_match = True
            self.NIP = None
        else:
            self.PIN = self.NIP


    def configure(self, cstatus='configured'):
        self.CSTATUS = cstatus
        self.write()
        

    def is_configured(self):
        return self.CSTATUS == 'configured'


    def toggle_sbar(self, state):
        if state == 'open':
            self.show_sbar = True
        else:
            self.show_sbar = False


    def hide_sidebar(self):
        self.hide_sbar = True


    def unhide_sidebar(self):
        self.hide_sbar = False


    def get_sidebar(self, sidebar='default'):
        if sidebar == 'config':
            self.sidebar_data = {
                'active' : self.show_sbar,
                'hidden' : self.hide_sbar,
                'method' : 'select_config_submenu',
                'title' : 'Settings',
                'icon' : 'fas fa-cogs',
                'submenu' : [
                    {
                        'name' : 'site',
                        'title' : 'Site Information',
                        'icon' : 'fas fa-hospital'
                    },
                    { 
                        'name' : 'studies',
                        'title' : 'Active Studies',
                        'icon' : 'fas fa-pills'
                    },
                    {
                        'name' : 'network',
                        'title' : 'Network',
                        'icon' : 'fas fa-wifi'
                    },
                    {
                        'name' : 'pin',
                        'title' : 'PIN Configuration',
                        'icon' : 'fas fa-lock'
                    }
                ]
            }
        else:
            self.sidebar_data = {
                'active' : self.show_sbar,
                'hidden' : self.hide_sbar,
                'method' : 'select_menu',
                'title' : '',
                'icon' : '',
                'submenu' : [
                    {
                        'name' : 'home',
                        'title' : 'Home',
                        'icon' : 'fas fa-home'
                    },
                    { 
                        'name' : 'record',
                        'title' : 'Record a Session',
                        'icon' : 'fas fa-video'
                    },
                    {
                        'name' : 'test',
                        'title' : 'Equipment Setup',
                        'icon' : 'fas fa-tasks'
                    },
                    {
                        'name' : 'config',
                        'title' : 'Settings',
                        'icon' : 'fas fa-cogs'
                    },
                    {
                        'name' : 'help',
                        'title' : 'Help',
                        'icon' : 'fas fa-info-circle'
                    }
                ]
            }
        return self.sidebar_data




class Program:
    """ Data class to store MDMA Program information"""

    COUNTRIES = {c.name : c.alpha2 for c in countries}

    STUDIES = ['MAPP1', 'MT1', 'MAPP2', 'MT2', 'MP18', 'MAPP3']

    SITES = {
        '00' : ('Santa Cruz', ['MAPP1', 'MT1', 'MAPP2', 'MT2', 'MP18', 'MAPP3']),
        '01' : ('Charleston', ['MAPP1', 'MT1']),
        '02' : ('Boulder', ['MAPP1', 'MT1']),
        '03' : ('Fort Collins', ['MAPP1']),
        '04' : ('Los Angeles', ['MAPP1']),
        '05' : ('New Orleans', ['MAPP1']),
        '06' : ('University of California, San Francisco', ['MAPP1']),
        '07' : ('San Francisco Insight and Integration Center', ['MAPP1']),
        '08' : ('University of Connecticut', ['MAPP1']),
        '09' : ('University of Wisconsin at Madison', ['MAPP1']),
        '10' : ('New York University', ['MAPP1']),
        '11' : ('Affective Care' , ['MAPP1']),
        '12' : ('Boston', ['MAPP1']),
        '13' : ('University of British Columbia', ['MAPP1']),
        '14' : ('Dr. Simon Amar, LLC', ['MAPP1']),
        '15' : ('Beer Yaakov', ['MAPP1']),
        '16' : ('Tel Hashomer', ['MAPP1']),
        '17' : ('Cardiff', ['MP18']),
        '18' : ('Leiden/Oegstgeest', ['MP18']),
        '19' : ('Maastricht', ['MP18']),
        '20' : ('Prague', ['MP18']),
        '21' : ('Bristol', ['MP18']),
        '22' : ('Portugal', ['MP18']),
        '23' : ('Norway', ['MP18']),
        '24' : ('Berlin', ['MP18']),
        '25' : ('Regensburg', ['MP18']),
        '26' : ('Finland', ['MP18'])
    }

    VISITS =  [
        'V01',
        'V02',
        'V04',
        'V05',
        'V06',
        'V07',
        'V09',
        'V10',
        'V11',
        'V12',
        'V14',
        'V15',
        'V16',
        'V17',
        'V18',
        'V20'
    ]

    PARTICIPANTS = {'{:02}{:02}0{:02}'.format(study, site, sub) for study in {11, 12, 13, 16, 17, 18} for site in range(1, 99) for sub in range(1, 99)}

    PARTICIPANTS = PARTICIPANTS.union({'TEST', 'PARTD', 'TOAD'})



